import os
import unittest
from unittest.mock import Mock, patch

from PyEmailerAJM import PyEmailer, Msg


class TestPyEmailer(unittest.TestCase):
    TEST_ATTACHMENT_NAMES = ['attachment1', 'attachment2']
    TEST_ADMIN_EMAIL = ['example@example.com']

    def setUp(self):
        PyEmailer.ADMIN_EMAIL = TestPyEmailer.TEST_ADMIN_EMAIL
        self.emailer = PyEmailer(False, False)

    @classmethod
    def tearDownClass(cls):
        for x in cls.TEST_ATTACHMENT_NAMES:
            try:
                os.remove(x)
            except FileNotFoundError:
                pass

    def test_init(self):
        self.assertIsInstance(self.emailer, PyEmailer)

    @unittest.skip("Skipping this test because it's under development")
    def test_current_user_email_getter_setter(self):
        self.emailer.current_user_email = 'test_user@example.com'
        self.assertEqual(self.emailer.current_user_email, 'test_user@example.com')

    def test_email_signature_getter(self):
        self.assertIsNone(self.emailer.email_signature)

    def test_send_success_getter_setter(self):
        self.emailer.send_success = True
        self.assertTrue(self.emailer.send_success)

    def test__display_tracking_warning_confirm(self):
        with patch('PyEmailerAJM.PyEmailer._display_tracking_warning_confirm', return_value=True):
            self.assertTrue(self.emailer._display_tracking_warning_confirm())

    def test_display_tracker_check(self):
        self.assertFalse(self.emailer.display_tracker_check())

    def test__GetReadFolder(self):
        # Create a single consistent Mock instance
        predefined_mock = Mock()

        # Use the patch to ensure the method returns the predefined_mock
        with patch('PyEmailerAJM.PyEmailer._GetReadFolder', return_value=predefined_mock):
            # Compare _GetReadFolder's return value with the same predefined_mock
            self.assertEqual(self.emailer._GetReadFolder(6), predefined_mock)

    def test_GetMessages(self):
        predefined_mock = Mock()  # Create a single consistent mock for the return value
        with patch('PyEmailerAJM.PyEmailer.GetMessages', return_value=predefined_mock):
            self.assertEqual(self.emailer.GetMessages(0), predefined_mock)  # Compare with the predefined mock

    def test_GetEmailMessageBody(self):
        msg = Mock()  # Create the message mock
        predefined_mock = Mock()  # Create a single consistent mock for the return value
        with patch('PyEmailerAJM.PyEmailer.GetEmailMessageBody', return_value=predefined_mock):
            result = self.emailer.GetEmailMessageBody(msg)
            self.assertEqual(result, predefined_mock)  # Compare with the predefined mock

    def test_FindMsgBySubject(self):
        subject = 'Test Subject'
        predefined_mock = Mock()  # Define a single Mock instance for consistent comparison

        # Patch the method to return the predefined_mock object
        with patch('PyEmailerAJM.PyEmailer.FindMsgBySubject', return_value=predefined_mock):
            result = self.emailer.FindMsgBySubject(subject, True, True, False)
            self.assertEqual(result, predefined_mock)  # Compare with the predefined Mock instance

    def test_SaveAllEmailAttachments(self):
        # Create a mock message object with mocked attachments
        msg = Mock()
        attachment1 = Mock()
        attachment2 = Mock()

        # Assign Attachments as a list of the attachment mocks
        msg.Attachments = [attachment1, attachment2]

        # Mock the SaveAsFile method for each attachment
        attachment1.SaveAsFile = Mock()
        attachment2.SaveAsFile = Mock()

        # Set save directory path
        save_dir_path = '/path/to/save'

        # Call the method and check no exception is raised
        try:
            self.assertIsNone(self.emailer.SaveAllEmailAttachments(msg, save_dir_path))

            # Verify SaveAsFile was called with correct paths
            attachment1.SaveAsFile.assert_called_once()
            attachment2.SaveAsFile.assert_called_once()
        except Exception as e:
            self.fail(f"SaveAllEmailAttachments raised an exception: {e}")

    def test_SetupEmail(self):
        recipient = 'recipient@test.com'
        subject = 'Test email'
        text = 'This is a test email.'
        for x in self.__class__.TEST_ATTACHMENT_NAMES:
            with open(f'./{x}', 'w'):
                pass
        self.assertIsInstance(self.emailer.SetupEmail(recipient, subject, text,
                                                      self.__class__.TEST_ATTACHMENT_NAMES), Msg)

    def test_SendOrDisplay(self):
        mocked_return_value = Mock()  # Create a single Mock instance
        with patch('PyEmailerAJM.PyEmailer.SendOrDisplay', return_value=mocked_return_value):
            self.assertEqual(self.emailer.SendOrDisplay(True), mocked_return_value)

    @unittest.skip("Skipping this test because it's under development")
    def test__fmsg_is_no_info_or_err(self):
        info = Mock()
        self.assertFalse(PyEmailer._fmsg_is_no_info_or_err(info))

    def test_get_failed_sends(self):
        fail_string_marker = 'fail'
        mocked_return_value = Mock()  # Single Mock instance for consistent comparison
        with patch('PyEmailerAJM.PyEmailer.get_failed_sends', return_value=mocked_return_value):
            self.assertEqual(self.emailer.get_failed_sends(fail_string_marker, True), mocked_return_value)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch, MagicMock
import logging

from PyEmailerAJM.backend.logger import PyEmailerLogger


# FIXME: some of these tests are flakey
class TestPyEmailerLogger(unittest.TestCase):
    def setUp(self) -> None:
        self.logger = PyEmailerLogger()

    def test_call(self):
        self.assertIs(self.logger(), self.logger.logger)

    @patch('PyEmailerAJM.backend.logger.DupeDebugFilter')
    def test_add_dupe_debug_to_handler(self, mock_dupe_debug_filter):
        handler = MagicMock()

        # FIX: Ensure handler.level is an integer
        handler.level = logging.WARNING  # Set a valid logging level

        self.logger._add_dupe_debug_to_handler(handler)
        mock_dupe_debug_filter.assert_called_once()
        handler.addFilter.assert_called_once_with(mock_dupe_debug_filter.return_value)

    @unittest.skip("Skipping this test because it's under development")
    def test_set_logger_class(self):
        result = self.logger._set_logger_class()
        self.assertIs(result, self.logger)

    @patch('PyEmailerAJM.backend.logger.Logger')
    def test_initialize_logger(self, mock_logger):
        self.logger.initialize_logger(logger=mock_logger)
        self.assertFalse(mock_logger.propagate)

    @patch('PyEmailerAJM.backend.logger.OutlookEmailHandler')
    def test_setup_email_handler(self, mock_email_handler):
        mock_email_handler_instance = mock_email_handler.return_value
        mock_email_handler_instance.level = logging.ERROR

        with patch.object(self.logger.logger, 'addHandler', autospec=True) as mock_add_handler:
            self.logger.setup_email_handler(email_msg='Test email', logger_admins=['admin1@gmail.com'])

            self.assertEqual(mock_email_handler_instance.level, logging.ERROR)
            mock_email_handler.assert_called_once_with(
                email_msg='Test email',
                project_name=self.logger.project_name,
                logger_dir_path=self.logger.log_location,
                recipient=['admin1@gmail.com']
            )
            mock_email_handler_instance.setLevel.assert_called_once_with(logging.ERROR)
            mock_email_handler_instance.setFormatter.assert_called_once_with(
                self.logger.formatter)
            mock_add_handler.assert_called_once_with(mock_email_handler_instance)

    @patch('PyEmailerAJM.backend.logger.FileHandler')
    def test_add_filter_to_file_handler(self, mock_file_handler):
        mock_file_handler.level = logging.INFO  # FIX: Set a valid level
        self.logger._add_filter_to_file_handler(mock_file_handler)
        mock_file_handler.addFilter.assert_called_once()

    @patch('PyEmailerAJM.backend.logger.StreamHandler')
    def test_add_filter_to_stream_handler(self, mock_stream_handler):
        mock_stream_handler.level = logging.DEBUG  # FIX: Set a valid level
        self.logger._add_filter_to_stream_handler(mock_stream_handler)
        mock_stream_handler.addFilter.assert_called_once()

    def test_project_name_getter(self):
        self.assertEqual(self.logger.project_name, self.logger.project_name)

    def test_project_name_setter(self):
        self.logger.project_name = 'NewProject'
        self.assertEqual(self.logger.project_name, 'NewProject')

    @patch('PyEmailerAJM.backend.logger.StreamHandlerIgnoreExecInfo')
    def test_create_stream_handler(self, mock_stream_handler):
        self.logger.create_stream_handler(log_level_to_stream=logging.WARNING)
        mock_stream_handler.assert_called_once()


if __name__ == '__main__':
    unittest.main()



import logging
import unittest
from datetime import datetime, timedelta
from pathlib import Path
from unittest.mock import patch, Mock

from PyEmailerAJM.continuous_monitor.backend import SnoozeTracking


class TestSnoozeTracking(unittest.TestCase):
    TEST_JSON_PATH = Path("./test.json")

    def setUp(self):
        self.file_path = self.__class__.TEST_JSON_PATH

        self.logger = logging.getLogger('test_logger')
        self.subject = "Test Email Subject"
        self.snooze_time = datetime.now() + timedelta(days=2)
        self.email_entries = {
            self.subject: self.snooze_time.isoformat()
        }

        self.snooze_tracking = SnoozeTracking(self.file_path, logger=self.logger)

    @classmethod
    def tearDownClass(cls):
        cls.TEST_JSON_PATH.unlink()

    @unittest.skip("this test only works if the file "
                   "is NOT deleted by tearDownClass, "
                   "then the test is run again")
    @patch('json.load')
    @patch('builtins.open')
    def test_json_loaded(self, mock_open, mock_json_load):
        mock_json_load.return_value = self.email_entries
        loaded_json = self.snooze_tracking.json_loaded
        mock_open.assert_called_once_with(self.file_path, 'r')
        self.assertEqual(loaded_json, self.email_entries, 'Should load JSON data from file')

    @patch('json.load')
    @patch('builtins.open')
    def test_write_entry(self, mock_open, mock_json_load):
        mock_json_load.return_value = {}
        new_snooze_time = datetime.now() + timedelta(days=3)
        self.snooze_tracking.write_entry(self.subject, new_snooze_time)
        self.assertEqual(self.snooze_tracking.json_loaded[self.subject], new_snooze_time,
                         'Should write entry to json_loaded')

    def test_convert_datetime(self):
        non_dt_val = "non_datetime_value"
        dt_val = datetime.now()
        self.assertEqual(self.snooze_tracking._convert_datetime(non_dt_val), non_dt_val,
                         'Non datetime value should remain unchanged')
        self.assertEqual(self.snooze_tracking._convert_datetime(dt_val), dt_val.isoformat(),
                         'Datetime value should be iso formatted')

    @patch('json.dump')
    @patch('builtins.open')
    def test_save_json(self, mock_open, mock_json_dump):
        with patch.object(self.logger, 'info') as mock_info:  # Mock the logger.info method
            self.snooze_tracking._json_loaded = self.email_entries
            self.snooze_tracking.save_json()

            # Use the mocked `info` method with assert_called.
            mock_info.assert_called_with(f"json saved to {self.file_path}")

            mock_open.assert_called_once_with(self.file_path, 'w')
            mock_json_dump.assert_called_once()

    def test_read_entry(self):
        self.snooze_tracking._json_loaded = self.email_entries
        output_snooze_time = self.snooze_tracking.read_entry(self.subject)
        self.assertEqual(output_snooze_time, self.snooze_time, 'Should correctly read snooze time from json_loaded')

    def test_snooze_msgs(self):
        mock_msg_1 = Mock()
        mock_msg_1.subject = "msg_1"
        mock_msg_1.msg_snoozed = False
        mock_msg_1.msg_snoozed_time = datetime.now() + timedelta(days=3)

        mock_msg_2 = Mock()
        mock_msg_2.subject = "msg_2"
        mock_msg_2.msg_snoozed = True
        mock_msg_2.msg_snoozed_time = datetime.now() + timedelta(days=1)

        msg_list = [mock_msg_1, mock_msg_2]
        output_msg_list = self.snooze_tracking.snooze_msgs(msg_list)
        self.assertEqual(output_msg_list, msg_list, 'Should return the same list')
        self.assertTrue(output_msg_list[0].msg_snoozed, 'Non-snoozed message should be marked as snoozed')


if __name__ == "__main__":
    unittest.main()



#! python3
"""
py_emailer_ajm.py

install win32 with pip install pywin32
"""
# imports
from os import environ
from os.path import isfile, join, isdir
from tempfile import gettempdir

# install win32 with pip install pywin32
import win32com.client as win32

# This is installed as part of pywin32
# noinspection PyUnresolvedReferences
from pythoncom import com_error
from logging import Logger, basicConfig, StreamHandler, FileHandler, getLogger
from email_validator import validate_email, EmailNotValidError
import questionary
# this is usually thrown when questionary is used in the dev/Non Win32 environment
# noinspection PyProtectedMember
from prompt_toolkit.output.win32 import NoConsoleScreenBufferError

from PyEmailerAJM import (EmailerNotSetupError, DisplayManualQuit,
                          deprecated,
                          Msg, FailedMsg)
from PyEmailerAJM.backend import BasicEmailFolderChoices, PyEmailerLogger
from PyEmailerAJM.searchers import SubjectSearcher


class EmailerInitializer:
    DEFAULT_EMAIL_APP_NAME = 'outlook.application'
    DEFAULT_NAMESPACE_NAME = 'MAPI'

    def __init__(self, display_window: bool,
                 send_emails: bool, logger: Logger = None,
                 auto_send: bool = False,
                 email_app_name: str = DEFAULT_EMAIL_APP_NAME,
                 namespace_name: str = DEFAULT_NAMESPACE_NAME, **kwargs):
        if logger:
            self.logger = logger
        else:
            self._elog = PyEmailerLogger(**kwargs)
            self.logger = self._elog()
        # print("Dummy logger in use!")

        self.email_app_name = email_app_name
        self.namespace_name = namespace_name

        self.email_app, self.namespace, self.email = self.initialize_email_item_app_and_namespace()

        self.display_window = display_window
        self.auto_send = auto_send
        self.send_emails = send_emails

    def initialize_new_email(self):
        if hasattr(self, 'email_app') and self.email_app is not None:
            self.email = Msg(self.email_app.CreateItem(0), logger=self.logger)
            return self.email
        raise AttributeError("email_app is not defined. Run 'initialize_email_item_app_and_namespace' first")

    def initialize_email_item_app_and_namespace(self):
        try:
            email_app, namespace = self._setup_email_app_and_namespace()
            email = self.initialize_new_email()
        except com_error as e:
            self.logger.error(e, exc_info=True)
            raise e
        return email_app, namespace, email

    def _setup_email_app_and_namespace(self):
        self.email_app = win32.Dispatch(self.email_app_name)

        self.logger.debug(f"{self.email_app_name} app in use.")
        self.namespace = self.email_app.GetNamespace(self.namespace_name)

        self.logger.debug(f"{self.namespace_name} namespace in use.")
        return self.email_app, self.namespace


class PyEmailer(EmailerInitializer, SubjectSearcher):
    # the email tab_char
    tab_char = '&emsp;'
    signature_dir_path = join((environ['USERPROFILE']),
                              'AppData\\Roaming\\Microsoft\\Signatures\\')

    DisplayEmailSendTrackingWarning = "THIS TYPE OF SEND CANNOT BE DETECTED FOR SEND SUCCESS AUTOMATICALLY."
    FAILED_SEND_LOGGER_STRING = "{num} confirmed failed send(s) found in the last {recent_days_cap} day(s)."

    DEFAULT_TEMP_SAVE_PATH = gettempdir()
    VALID_EMAIL_FOLDER_CHOICES = [x for x in BasicEmailFolderChoices]

    def __init__(self, display_window: bool, send_emails: bool, logger: Logger = None, email_sig_filename: str = None,
                 auto_send: bool = False, email_app_name: str = EmailerInitializer.DEFAULT_EMAIL_APP_NAME,
                 namespace_name: str = EmailerInitializer.DEFAULT_NAMESPACE_NAME, **kwargs):

        super().__init__(display_window, send_emails, logger, auto_send, email_app_name, namespace_name, **kwargs)
        self._setup_was_run = False
        self._current_user_email = None

        self.read_folder = None

        self._email_signature = None
        self._send_success = False
        self.email_sig_filename = email_sig_filename

    @property
    def current_user_email(self):
        if self.email_app_name.lower().startswith('outlook'):
            self._current_user_email = (
                self.namespace.Application.Session.CurrentUser.AddressEntry.GetExchangeUser().PrimarySmtpAddress)
        return self._current_user_email

    @current_user_email.setter
    def current_user_email(self, value):
        try:
            if validate_email(value, check_deliverability=False):
                self._current_user_email = value
        except EmailNotValidError as e:
            self.logger.error(e, exc_info=True)
            value = None
        self._current_user_email = value

    @property
    def email_signature(self):
        return self._email_signature

    @email_signature.getter
    def email_signature(self):
        if self.email_sig_filename:
            signature_full_path = join(self.signature_dir_path, self.email_sig_filename)
            if isdir(self.signature_dir_path):
                pass
            else:
                try:
                    raise NotADirectoryError(f"{self.signature_dir_path} does not exist.")
                except NotADirectoryError as e:
                    self.logger.warning(e)
                    self._email_signature = None

            if isfile(signature_full_path):
                with open(signature_full_path, 'r', encoding='utf-16') as f:
                    self._email_signature = f.read().strip()
            else:
                try:
                    raise FileNotFoundError(f"{signature_full_path} does not exist.")
                except FileNotFoundError as e:
                    self.logger.warning(e)
                    self._email_signature = None
        else:
            self._email_signature = None
            self.logger.info("email_sig_filename not specified, no email signature will be attached.")

        return self._email_signature

    @property
    def send_success(self):
        return self._send_success

    @send_success.setter
    def send_success(self, value):
        self._send_success = value

    def _display_tracking_warning_confirm(self):
        # noinspection PyBroadException
        try:
            q = questionary.confirm(f"{self.DisplayEmailSendTrackingWarning}. Do you understand?",
                                    default=False, auto_enter=False).ask()
            return q
        except Exception as e:
            # TODO: slated for removal
            # this is here purely as a compatibility thing, to be taken out later.
            self.logger.warning(e)
            self.logger.warning("Defaulting to basic y/n prompt.")
            while True:
                q = input(f"{self.DisplayEmailSendTrackingWarning}. Do you understand? (y/n): ").lower().strip()
                if q == 'y':
                    self.logger.warning(self.DisplayEmailSendTrackingWarning)
                    return True
                elif q == 'n':
                    return False
                else:
                    print("Please respond with 'y' or 'n'.")

    def display_tracker_check(self) -> bool:
        if self.display_window:
            c = self._display_tracking_warning_confirm()
            if c:
                return c
            else:
                try:
                    raise DisplayManualQuit("User cancelled operation due to DisplayTrackingWarning.")
                except DisplayManualQuit as e:
                    self.logger.error(e, exc_info=True)
                    raise e

    def _get_default_folder_for_email_dir(self, email_dir_index: int = None, **kwargs):
        # 6 = inbox
        if email_dir_index in self.__class__.VALID_EMAIL_FOLDER_CHOICES:
            self.read_folder = self.namespace.GetDefaultFolder(email_dir_index)
            return self.read_folder
        else:
            try:
                raise ValueError(f"email_dir_index must be one of {self.__class__.VALID_EMAIL_FOLDER_CHOICES}")
            except ValueError as e:
                self.logger.error(e, exc_info=True)
                raise e

    def _GetReadFolder(self, email_dir_index: int = None, **kwargs):
        """
        :param email_dir_index: Specifies the email directory index to be accessed. Defaults to None.
        :type email_dir_index: int, optional
        :param kwargs: Additional optional arguments that may be passed. Can include `subfolder_name` to specify a subfolder name, defaulting to 'Inbox'.
        :type kwargs: dict
        :return: The folder specified either by the email directory index or the default folder along with the subfolder if applicable.
        :rtype: object
        """
        subfolder_name = kwargs.get('subfolder_name', 'Inbox')
        if not email_dir_index:
            email_dir_index = BasicEmailFolderChoices.INBOX
            self.logger.debug(f">>> email_dir_index not specified, defaulting to '{email_dir_index}' folder. <<<")
        if not isinstance(email_dir_index, int):
            self.logger.debug(f">>> email_dir_index is not an int, "
                               f"defaulting to {email_dir_index} folder and {subfolder_name} subfolder. <<<")
            return self.namespace.Folders[email_dir_index].Folders[subfolder_name]

        else:
            return self._get_default_folder_for_email_dir(email_dir_index)

    def GetMessages(self, folder_index=None):
        if isinstance(folder_index, int):
            self.read_folder = self._GetReadFolder(folder_index)
        elif not folder_index and self.read_folder:
            pass
        elif not folder_index:
            self.read_folder = self._GetReadFolder()
        else:
            try:
                raise TypeError("folder_index must be an integer or self.read_folder must be defined")
            except TypeError as e:
                self.logger.error(e, exc_info=True)
                raise e
        return [Msg(m, logger=self.logger) for m in self.read_folder.Items]

    @deprecated("use Msg classes body attribute instead")
    def GetEmailMessageBody(self, msg):
        """message = messages.GetLast()"""
        body_content = msg.body
        if body_content:
            return body_content
        else:
            try:
                raise ValueError("This message has no body.")
            except ValueError as e:
                self.logger.error(e, exc_info=True)
                raise e

    @deprecated("use find_messages_by_subject instead")
    def FindMsgBySubject(self, subject: str, forwarded_message_match: bool = True,
                         reply_msg_match: bool = True, partial_match_ok: bool = False):
        return self.find_messages_by_subject(subject, include_fw=forwarded_message_match,
                                             include_re=reply_msg_match,
                                             partial_match_ok=partial_match_ok)

    def SaveAllEmailAttachments(self, msg, save_dir_path):
        attachments = msg.Attachments
        for attachment in attachments:
            full_save_path = join(save_dir_path, str(attachment))
            try:
                attachment.SaveAsFile(full_save_path)
                self.logger.debug(f"{full_save_path} saved from email with subject {msg.subject}")
            except Exception as e:
                self.logger.error(e, exc_info=True)
                raise e

    def SetupEmail(self, recipient: str, subject: str, text: str, attachments: list = None, **kwargs):
        self.email = self.email.SetupMsg(sender=self.current_user_email, email_item=self.email(),
                                         recipient=recipient, subject=subject, body=text, attachments=attachments,
                                         logger=self.logger, **kwargs)
        self._setup_was_run = True
        return self.email

    def _manual_send_loop(self):
        try:
            send = questionary.confirm("Send Mail?:", default=False).ask()
            if send:
                self.email.send()
                return
            elif not send:
                self.logger.info(f"Mail not sent to {self.email.to}")
                print(f"Mail not sent to {self.email.to}")
                q = questionary.confirm("do you want to quit early?", default=False).ask()
                if q:
                    print("ok quitting!")
                    self.logger.warning("Quitting early due to user input.")
                    exit(-1)
                else:
                    return
        except com_error as e:
            self.logger.error(e, exc_info=True)
        except NoConsoleScreenBufferError as e:
            # TODO: slated for removal
            # this is here purely as a compatibility thing, to be taken out later.
            self.logger.warning(e)
            self.logger.warning("defaulting to basic input style...")
            while True:
                yn = input("Send Mail? (y/n/q): ").lower()
                if yn == 'y':
                    self.email.send()
                    break
                elif yn == 'n':
                    self.logger.info(f"Mail not sent to {self.email.to}")
                    print(f"Mail not sent to {self.email.to}")
                    break
                elif yn == 'q':
                    print("ok quitting!")
                    self.logger.warning("Quitting early due to user input.")
                    exit(-1)
                else:
                    print("Please choose \'y\', \'n\' or \'q\'")

    def SendOrDisplay(self, print_ready_msg: bool = False):
        if self._setup_was_run:
            if print_ready_msg:
                print(f"Ready to send/display mail to/for {self.email.to}...")
            self.logger.info(f"Ready to send/display mail to/for {self.email.to}...")
            if self.send_emails and self.display_window:
                send_and_display_warning = ("Sending email while also displaying the email "
                                            "in the app is not possible. Defaulting to Display only")
                # print(send_and_display_warning)
                self.logger.warning(send_and_display_warning)
                self.send_emails = False
                self.display_window = True

            if self.send_emails:
                if self.auto_send:
                    self.logger.info("Sending emails with auto_send...")
                    self.email.send()

                else:
                    self._manual_send_loop()

            elif self.display_window:
                self.email.display()
            else:
                both_disabled_warning = ("Both sending and displaying the email are disabled. "
                                         "No errors were encountered.")
                self.logger.warning(both_disabled_warning)
                # print(both_disabled_warning)
        else:
            try:
                raise EmailerNotSetupError("Setup has not been run, sending or displaying an email cannot occur.")
            except EmailerNotSetupError as e:
                self.logger.error(e, exc_info=True)
                raise e

    @staticmethod
    def _fmsg_is_no_info_or_err(info):
        return (any(isinstance(x, Exception) for x in info)
                or all(isinstance(x, type(None)) for x in info))

    def get_failed_sends(self, fail_string_marker: str = 'undeliverable', partial_match_ok: bool = True, **kwargs):
        failed_sends = []
        recent_days_cap = kwargs.get('recent_days_cap', 1)
        self.GetMessages(BasicEmailFolderChoices.INBOX)

        msg_candidates = self.FindMsgBySubject(fail_string_marker, partial_match_ok=partial_match_ok)

        if msg_candidates:
            msg_candidates = [FailedMsg(m) for m in msg_candidates]
            self.logger.info(f"{len(msg_candidates)} 'failed send' candidates found.")
            self.logger.info("mutating msg_candidates (Msg instances) into FailedMsg instances.")

            for m in msg_candidates:
                failed_info = m.process_failed_msg(m(), recent_days_cap=recent_days_cap)

                if self._fmsg_is_no_info_or_err(failed_info):
                    continue
                else:
                    failed_sends.append({'postmaster_email': m.sender,
                                         'err_info': failed_info})
        results_string = self.__class__.FAILED_SEND_LOGGER_STRING.format(num=len(failed_sends),
                                                                         recent_days_cap=recent_days_cap)
        if (not self.logger.hasHandlers() or not any([isinstance(x, StreamHandler)
                                                       for x in self.logger.handlers])):
            print(results_string)
        self.logger.info(results_string)
        return failed_sends


def __failed_sends_test(emailer):
    failed_sends = emailer.get_failed_sends(recent_days_cap=1)
    fs_results = ([(x.get('err_info').get('send_time'),
                    x.get('err_info').get('failed_subject'))
                   for x in failed_sends]
                  if failed_sends else "no failed sends found")
    print(fs_results)


def __setup_and_send_test(emailer):
    emailer.SetupEmail(subject="TEST: Your TEST agreement expires in 30 days or less!",
                       recipient='amcsparron@albanyny.gov',
                       text="testing to see anything works", bcc='amcsparron@albanyny.gov')
    emailer.SendOrDisplay()


if __name__ == "__main__":
    module_name = __file__.split('\\')[-1].split('.py')[0]

    em = PyEmailer(display_window=False, send_emails=True, auto_send=False, use_default_logger=False)
    m = em.find_messages_by_subject('Andrew', partial_match_ok=True)
    print([type(x) for x in m])
    # __setup_and_send_test(em)
    # __failed_sends_test(em)
    x = em.find_messages_by_subject("GIS Request", partial_match_ok=True)
    # [x.name for x in m.ItemProperties]
    print([(m.__class__, m.sender, m.sender_email_type, m.subject)
           for m in [Msg(y) for y in x]])  # for m in x])
    # property_accessor = x[0].PropertyAccessor
    # print(x[0].Sender.GetExchangeUser().PrimarySmtpAddress)
    # print(property_accessor.GetProperty("PR_EMAIL_ADDRESS"))

    # r_dict = {
    #     "subject": f"TEST: Your TEST "
    #                f"agreement expires in 30 days or less!",
    #     "text": "testing to see if the attachment works",
    #     "recipient": 'test',
    #     "attachments": []
    # }
    # # &emsp; is the tab character for emails
    # emailer.SetupEmail(**r_dict)  # recipient="test", subject="test subject", text="test_body")
    # emailer.SendOrDisplay()



from pathlib import Path
from importlib import import_module
# TODO: add these to other projects? and or EasyLogger?
__project_root__ = Path(__file__).parent.parent
__project_name__ = __project_root__.name
print(f'logs for {__project_name__} found in {__project_root__ / "logs"}')


def is_instance_of_dynamic(obj: object, base_class_path: str) -> bool:
    """
    Check if an object is an instance of a class or its subclass specified by its module path.
    """
    try:
        module_path, class_name = base_class_path.rsplit('.', 1)
        module = import_module(module_path)
        base_class = getattr(module, class_name)
        return isinstance(obj, base_class)
    except (ImportError, AttributeError):
        return False


from PyEmailerAJM.backend import deprecated
from PyEmailerAJM.backend.errs import EmailerNotSetupError, DisplayManualQuit
from PyEmailerAJM.msg import Msg, FailedMsg
from PyEmailerAJM.searchers import BaseSearcher, SubjectSearcher
from PyEmailerAJM.py_emailer_ajm import PyEmailer, EmailerInitializer
from PyEmailerAJM.continuous_monitor.continuous_monitor import ContinuousMonitor

__all__ = ['EmailerNotSetupError', 'DisplayManualQuit', 'deprecated',
           'Msg', 'FailedMsg', 'PyEmailer', 'EmailerInitializer',
           'BaseSearcher', 'SubjectSearcher', 'ContinuousMonitor',
           '__project_root__', '__project_name__', 'is_instance_of_dynamic']




from setuptools import setup
import re

project_name = 'PyEmailerAJM'


def get_property(prop, project):
    result = re.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop),
                       open(project + '/_version.py').read())
    return result.group(1)


setup(
    name=project_name,
    version=get_property('__version__', project_name),
    packages=['PyEmailerAJM'],
    url='https://github.com/amcsparron2793-Water/PyEmailer',
    download_url=f'https://github.com/amcsparron2793-Water/PyEmailer/archive/refs/tags/{get_property("__version__", project_name)}.tar.gz',
    keywords=["Outlook", "Email", "Automation"],
    install_requires=['pywin32', 'extract_msg', 'email_validator', 'questionary','EasyLoggerAJM', 'ColorizerAJM'],
    license='MIT License',
    author='Amcsparron',
    author_email='amcsparron@albanyny.gov',
    description='Allows for automating sending Email with the Outlook Desktop client.'
                ' Future releases will add more client support'
)
